#ifndef _LITTLE_ERROR_H_
#define _LITTLE_ERROR_H_
/*
 * The LITTLE runtime error handling routines.
 */
#include <little/little.h>
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Makes sure, the class identified by 'name', is properly referred by
 * 'sym'.
 * Arguments:
 * - ltl_sym_t sym
 * - const char *name
 * Example:
 * ...
 * static ltl_sym_t myclass = -1;
 * LTL_ASSERT_CLASS(myclass,"mypackage/MyClass");
 * ...
 */
#define LTL_ASSERT_CLASS(sym,name) \
	if ( sym < 0 ) { \
		sym = ltl_threadenv_getsym(threadenv,name); \
		ltl_threadenv_get_global4sym(threadenv,sym); \
	}

/*
 * Use this macro in native method definitions, to check call argument for type conformance.
 * If a mismatch is found, Error 'TypeMismatch' is raised.
 * Arguments:
 * - int n
 * - ltl_sym_t t
 * - const char *methodname
 */
#define LTL_CHECK_ARG(n,t,methodname) \
	if ( ! ltl_threadenv_conjunct(threadenv,(t),argv[n].type) ) {\
		printf("CheckArg: %s arg #%d: <%s> instead of <%s>\n",\
			(methodname),\
			(n),\
			ltl_threadenv_get_str4sym(threadenv,argv[n].type),\
			ltl_threadenv_get_str4sym(threadenv,(t)));\
		LITTLE_TYPEMISMATCH(\
			threadenv,\
			ltl_threadenv_get_str4sym(threadenv,(t)),\
			ltl_threadenv_get_str4sym(threadenv,argv[n].type));\
	}	

void little_perror(ltl_threadenv_t *,FILE *);

/*
 *
 */
void little_runtime_error(
	ltl_threadenv_t *,
	ltl_sym_t,
	const char *,
	const char *);


#define LITTLE_ERROR(threadenv,token,msg1,msg2) \
	little_runtime_error((threadenv),ltl_threadenv_getsym(threadenv,(token)),(msg1),(msg2))

#define LITTLE_TYPEMISMATCH(threadenv,msg1,msg2) \
	LITTLE_ERROR((threadenv),"TypeMismatch",(msg1),(msg2))

#ifdef __cplusplus
}
#endif

#endif
